classdef FunctionSystem < handle
    
    properties(SetAccess = protected)
        
        %(Optional) cell of Functional-objects to determine
        %linear independence
        functionals
        
        %Number of functions that create the function system
        N
    end
    
    methods
        function setFunctionals(obj, varargin)
            if ~all(cellfun(@(L) isa(L, 'Functional'), varargin))
                error('All arguments must be function handles');
            end
            
            if length(varargin) ~= obj.N
                error('The functional system has the wrong length.');
            end
            
            obj.functionals = varargin;
        end
        
        function b = isIndependent(obj)
            if isempty(obj.functionals)
                b = false;
                return
            end
            
            ev = @(L) arrayfun(@(i) L.of(obj.get(i)), 1:obj.N);
            A = cellfun(@(L) ev(L), obj.functionals, 'UniformOutput', false);
            A = vertcat(A{:});
            
            b = (rank(A) == obj.N);
        end
    end
    
    methods(Abstract)
        
        %This function takes a number x in (0,1) and returns a column
        %vector [phi_1(x) ... phi_n(x)]', where the phi_i are the basis
        %functions of the function system.
        evaluate(obj, x)
        
        get(obj, i)
    end
end
